<?php
# @Author: Aaron Bowie
# @Date:   Saturday, September 16th 2017, 12:09:42 am
# @Email:  support@weareag.co.uk
# @Filename: ag-woocommerce-adyen-class.php
# @Last modified by:   Aaron Bowie
# @Last modified time: Tuesday, February 6th 2018, 3:59:32 pm


defined('ABSPATH') or die("No script kiddies please!");


/*-----------------------------------------------------------------------------------*/
/*  Set class
/*-----------------------------------------------------------------------------------*/

class ag_adyen extends WC_Payment_Gateway {

function __construct() {

	$this->id = "ag_adyen";
	$this->method_title = __( "AG Adyen HPP", 'ag-adyen' );
  $license = get_option('AGD_adyen_Key');
  $websiteurl = get_option ('siteurl');
  $api_params = array(
    'slm_action' => 'slm_check',
    'secret_key' => AGD_KEY,
    'license_key' => $license,
  );
  $query = esc_url_raw(add_query_arg($api_params, AGD));
  $response = wp_remote_get($query, array('timeout' => 20, 'sslverify' => false));
  $license_data = json_decode(wp_remote_retrieve_body($response));

  if($license_data->result == 'success'){
    $this->method_description =  __( "Adyen HPP payment gateway plugin for WooCommerce", 'ag-adyen' );
  } else {
    $this->method_description = __( 'Please enter the license key for this product to activate it. You were given a license key when you purchased this item. <br /><strong>You can do this on the AG Plugins menu item on the left</strong>. ', 'ag-adyen' );
  }

	$this->title = __( "Adyen HPP", 'ag-adyen' );
	$this->icon = plugins_url('AG-Woo-Adyen/img/weareag.woo.adyen-logo.png');
	$this->has_fields = false;

	$this->environment 		= $this->get_option('environment');


	// Supports
	$this->supports = array( 'refunds' );

  // actions
	add_action('woocommerce_thankyou_ag_adyen', array($this, 'thank_you'));
	add_action( 'woocommerce_api_woocommerce_ag_adyen', array( $this, 'successful_request' ) );
	add_action( 'woocommerce_receipt_ag_adyen', array( $this, 'receipt_page' ) );




  if($license_data->result == 'success'){
    $this->init_form_fields();
    $this->init_settings();
  }

	foreach ( $this->settings as $setting_key => $value ) {
		$this->$setting_key = $value;
	}

	// Save settings
	if ( is_admin() ) {
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
	}
} // End __construct()

/*-----------------------------------------------------------------------------------*/
/*  Build setting fields
/*-----------------------------------------------------------------------------------*/

public function init_form_fields() {

  $this->form_fields = array(
		'enabled'   => array(
			'title'		=> __( 'Enable / Disable', 'ag-adyen' ),
			'label'		=> __( 'Enable this payment gateway', 'ag-adyen' ),
			'type'		=> 'checkbox',
			'default'	=> 'no',
		),
		'title' => array(
			'title'		=> __( 'Title', 'ag-adyen' ),
			'type'		=> 'text',
			'desc_tip'	=> __( 'Payment title the customer will see during the checkout process.', 'ag-adyen' ),
			'default'	=> __( 'Credit card payments by Adyen', 'ag-adyen' ),
		),
		'description' => array(
			'title'		  => __( 'Description', 'ag-adyen' ),
			'type'		  => 'textarea',
			'desc_tip'	=> __( 'Payment description the customer will see during the checkout process.', 'ag-adyen' ),
			'default'	  => __( 'Pay securely using your credit card.', 'ag-adyen' ),
			'css'		    => 'max-width:350px;'
		),
    'merchantAccount' => array(
      'title'		  => __( 'Adyen Merchant Account', 'ag-adyen' ),
      'type'		  => 'text',
      'desc_tip'	=> __( 'This is the Merchant Account provided by adyen when you signed up for an account.', 'ag-adyen' ),
    ),
		'skinCode'         => array(
        'title'         => __( 'Skin Code', 'woocommerce_ag_adyen' ),
        'type'          => 'text',
        'description'   => __( 'You can create, edit, upload, test, and publish your skins on the Skins tab in the Customer Area. <br />See this link for more <a target="_blank" href="https://docs.adyen.com/developers/features/hosted-payment-pages/skin">information</a>' ),
    ),
		'HmacKey'         => array(
				'title'         => __( 'HMAC Key', 'woocommerce_ag_adyen' ),
				'type'          => 'text',
				'description'   => __( 'The secret HMAC key is used to calculate the request signature and is linked to a skin.<br />See this link for more <a target="_blank" href="https://docs.adyen.com/developers/user-management/how-to-get-hmac-keys-for-a-skin">information</a>' ),
		),

    'ship' => array(
      'title' => __('Shipping days', 'ag-adyen'),
      'type' => 'select',
      'css'  => 'height: 35px;',
      'options' => array(
        '1' => __('1 Day', 'ag-adyen'),
        '2' => __('2 Days', 'ag-adyen'),
        '3' => __('3 Days', 'ag-adyen'),
				'4' => __('4 Days', 'ag-adyen'),
				'5' => __('5 Days', 'ag-adyen'),
				'6' => __('6 Days', 'ag-adyen')

      ),
      'description' => __('Select number of days before order is shipped.', 'ag-adyen')
    ),

		'environment' => array(
			'title'		=> __( 'Adyen Mode', 'ag-adyen' ),
			'label'		=> __( 'Enable Mode', 'ag-adyen' ),
			'type' => 'select',
			'css'  => 'height: 35px;',
			'options'=> array('test'=>'Test Environment','live'=>'Live Store'),
			'description' => __( 'The status of your store tells that are you actually ready to run your shop or its still a test environment.', 'ag-adyen' ),
		),


		'user' => array(
			'title'		=> __( 'Username', 'ag-adyen' ),
			'type'		=> 'text',
			'description' => __( 'This is used for processing refunds only, <strong>NOT</strong> needed to process payments.<br />Username format is <strong>ws@Company.[YourCompanyAccount]</strong>', 'ag-adyen' ),
		),
		'password' => array(
			'title'		=> __( 'Password', 'ag-adyen' ),
			'type'		=> 'password',
			'description' => __( 'This is used for processing refunds only, <strong>NOT</strong> needed to process payments.<br />To set the corresponding password, go to the Customer Area, then select Settings > Users.', 'ag-adyen' ),
		),

	);
}





/*-----------------------------------------------------------------------------------*/
/*	Process the payment and return the result
/*-----------------------------------------------------------------------------------*/
function process_payment($order_id){

	$order = new WC_Order( $order_id );

	return array(
		'result' 	=> 'success',
		'redirect'	=> $order->get_checkout_payment_url( true )
	);

}

/*-----------------------------------------------------------------------------------*/
/*	receipt_page
/*-----------------------------------------------------------------------------------*/
function receipt_page($order_id){
	global $woocommerce;
	$order = new WC_Order( $order_id );

	$skinCode        = $this->skinCode;
  $merchantAccount = $this->merchantAccount;
 	$hmacKey         = $this->HmacKey;
	$expire = date("c", strtotime('+1 hour'));
	$shipdate = date("c", strtotime('+'.$this->ship.' day'));
	$notify_url = $order->get_checkout_order_received_url();

	$params = array(
									"merchantReference" => $this->prefix . $order->id,
									"merchantAccount"   => $merchantAccount,
									"currencyCode"      => get_woocommerce_currency(),
									"paymentAmount"     => $order->order_total*100,
									"sessionValidity"   => $expire,
									"shipBeforeDate"    => $shipdate,
									"shopperLocale"     => "en_GB",
									"skinCode"          => $skinCode,
									"shopperEmail"      => $order->billing_email,
									"shopperReference"  => $this->prefix . $order->id,
									"landingPage"			 => "onePage",
									"shopper.firstName" => $order->billing_first_name,
									"shopper.lastName" => $order->billing_last_name,
									"shopper.telephoneNumber" => $order->billing_phone,
									"billingAddress.street" => $order->billing_address_1,
									"billingAddress.city" => $order->billing_city,
									"billingAddress.stateOrProvince" => $order->billing_state,
									"billingAddress.postalCode" => $order->billing_postcode,
									"resURL" => $notify_url


	);


	$escapeval = function($val) {
			return str_replace(':','\\:',str_replace('\\','\\\\',$val));
	};

	ksort($params, SORT_STRING);

	$signData = implode(":",array_map($escapeval,array_merge(array_keys($params), array_values($params))));

	// base64-encode the binary result of the HMAC computation
	$merchantSig = base64_encode(hash_hmac('sha256',$signData,pack("H*" , $hmacKey),true));
	$params["merchantSig"] = $merchantSig;

	$environment = $this->environment;

	if($environment == "test") {
		$url = 'https://test.adyen.com/hpp/select.shtml';
	} elseif ($environment == "live") {
		$url = 'https://live.adyen.com/hpp/select.shtml';
	}

		echo '<p>'.__('Thank you for your order, please click the button below to pay securely.', 'ag-adyen').'</p>';
		echo '<form action="'.$url.'" method="post" name="adyenForm" id="adyenForm">';
		foreach ($params as $key => $value){
				echo '<input type="hidden" name="' .htmlspecialchars($key,   ENT_COMPAT | ENT_HTML401 ,'UTF-8').
				'" value="' .htmlspecialchars($value, ENT_COMPAT | ENT_HTML401 ,'UTF-8') . '" />' ."\n" ;
		}
		echo '<input type="submit" class="button alt" id="submit_adyenForm" value="'.__('Pay securely', 'ag-adyen').'" />';
		echo '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
		echo '<a class="button cancel" href="'.$order->get_cancel_order_url().'">'.__('Cancel order &amp; restore cart', 'ag-adyen').'</a></form>';


	wc_enqueue_js('
		jQuery("body").block({
				message: "'.__('You are now redirecting to Adyen server to make payment securly.', 'ag-adyen').'",
				overlayCSS:
				{
					background: "#fff",
					opacity: 0.6
				},
				css: {
							padding:        20,
							textAlign:      "center",
							color:          "#555",
							border:         "3px solid #aaa",
							backgroundColor:"#fff",
							cursor:         "wait",
							lineHeight:		"32px"
					}
			});
		jQuery("#submit_adyenForm").click();
	');
}

/*-----------------------------------------------------------------------------------*/
/*	Successful Payment WOOP WOOP!
/*-----------------------------------------------------------------------------------*/
function successful_request() {}

function thank_you() {
	global $woocommerce;

	if ( !$_REQUEST['merchantReference'] )
		return;
	$order = new WC_Order( $_REQUEST['merchantReference'] );

	$result = $_REQUEST['authResult'];

	if (!empty($_REQUEST['authResult']))
			$note = 'Order Status: ' . $_REQUEST['authResult'] . '.<br><br>';
	if (!empty($_REQUEST['merchantSig']))
			$note .= 'merchantSig: ' . $_REQUEST['merchantSig'] . '.<br><br>';
	if (!empty($_REQUEST['paymentMethod']))
			$note .= 'Payment Method: ' . $_REQUEST['paymentMethod'] . '.<br><br>';
	if (!empty($_REQUEST['pspReference']))
			$note .= 'PSP Reference: ' . $_REQUEST['pspReference'] . '.<br><br>';
	if (!empty($_REQUEST['shopperLocale']))
			$note .= 'Shoper Location: ' . $_REQUEST['shopperLocale'] . '.<br><br>';
	if (!empty($_REQUEST['key']))
			$note .= 'WC Order Key: ' . $_REQUEST['key'] . '.<br><br>';

	// store for later
	if (isset($_REQUEST['merchantSig'])) {
		update_post_meta($order->id, 'merchantSig', $_REQUEST['merchantSig']);
	}
	if (isset($_REQUEST['pspReference'])) {
		update_post_meta($order->id, 'pspReference', $_REQUEST['pspReference']);
	}
	if (isset($_REQUEST['key'])) {
		update_post_meta($order->id, 'wc_order_key', $_REQUEST['key']);
	}


	switch ($result):

		case AUTHORISED:
			$order->add_order_note($note);
			$noteTitle = 'Adyen transaction is confirmed.';
			$order->add_order_note($noteTitle);
			$order->payment_complete();
			$woocommerce->cart->empty_cart();
			break;

		case REFUSED:
			$dienote .= '<br>The payment was refused. Payment authorisation was unsuccessful.';
			$order->update_status('failed', $dienote);
			$woocommerce->cart->empty_cart();
			break;

		case CANCELLED:
			$dienote .= '<br>The payment was cancelled by the shopper before completion, or the shopper returned to the merchants site before completing the transaction.';
			$order->update_status('failed', $dienote);
			$woocommerce->cart->empty_cart();
			break;

		case PENDING:
			$dienote .= '<br>It is not possible to obtain the final status of the payment.
This can happen if the systems providing final status information for the payment are unavailable, or if the shopper needs to take further action to complete the payment.';
			$order->update_status('on-hold', $dienote);
			$woocommerce->cart->empty_cart();
			break;

		case ERROR:
			$dienote .= '<br>An error occurred during the payment processing.';
			$order->update_status('failed', $dienote);
			$woocommerce->cart->empty_cart();
			break;

		endswitch;

}

/*-----------------------------------------------------------------------------------*/
/*	Refunds
/*-----------------------------------------------------------------------------------*/

public function process_refund( $order_id, $amount = NULL, $reason = '' ) {

	$order 			   = new WC_Order( $order_id );


	$environment = $this->environment;

	if($environment == "test") {
		$environment_url = 'https://pal-test.adyen.com/pal/servlet/Payment/v30/refund';
	} elseif ($environment == "live") {
		$environment_url = 'https://pal-live.adyen.com/pal/servlet/Payment/v30/refund';
	}


	$ref = get_post_meta( $order_id, 'pspReference', true );
	$currency = get_woocommerce_currency();


	$actual_string = 'merchantAccount='.$this->merchantAccount.'&originalReference='.$ref.'&reference='.$reason.'&modificationAmount.currency='.$currency.'&modificationAmount.value='.$amount*100;

	$result = wp_remote_post($environment_url, array(
		'headers' => array(
        'Authorization' => 'Basic ' . base64_encode( $this->user . ':' . $this->password ),
    ),
		'body' => $actual_string,
	));

	$lines = preg_split('/\r\n|\r|\n/', $result['body']);
	$response = array();
	foreach ($lines as $line) {
			$key_value = preg_split('/=/', $line, 2);
			if (count($key_value) > 1) {
					$response[trim($key_value[0])] = trim($key_value[1]);
			}
	}

	$string = implode('|',$response);

	$refund_psp = $response['pspReference'];
	$refund_response = $response['response'];

	//if (!is_wp_error($result) && $result['response']['code'] >= 200 && $result['response']['code'] < 300) {


		if ($refund_response = '[refund-received]') {
			$order->add_order_note( __('Refund successful', 'ag-adyen') . '<br />' .
				__('Refund Amount: ', 'ag-adyen') . $amount . '<br />' .
				__('Refund Reason: ', 'ag-adyen') . $reason . '<br />'

		);

		return true;


		} else {
			$order->add_order_note(
				__('Refund failed', 'ag-adyen') . '<br />' .
				__('return code: ', 'ag-adyen') . $string . '<br />'

			);

		}

	//}

}



}
